@@ -51,11 +51,9 @@
 #'
 #' @export
 multicore <- function(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, workers = availableCores(constraints = "multicore"), earlySignal = FALSE, label = NULL, ...) {
-  if ("maxCores" %in% names(list(...))) {
-    .Defunct(msg = "Argument 'maxCores' has been renamed to 'workers'. Please update you script/code that uses the future package.")
-  }
-
   if (substitute) expr <- substitute(expr)
+
+  if (is.function(workers)) workers <- workers()
   workers <- as.integer(workers)
   stopifnot(is.finite(workers), workers >= 1L)
 
