@@ -18,11 +18,15 @@ nbrOfWorkers <- function(evaluator = NULL) {
 nbrOfWorkers.cluster <- function(evaluator) {
   expr <- formals(evaluator)$workers
   workers <- eval(expr)
-  stopifnot(is.character(workers) || is.numeric(workers) || inherits(workers, "cluster"), !anyNA(workers))
+  if (is.function(workers)) workers <- workers()
   if (is.character(workers)) {
+    stopifnot(!anyNA(workers))
     workers <- length(workers)
+  } else if (is.numeric(workers)) {
   } else if (inherits(workers, "cluster")) {
     workers <- length(workers)
+  } else {
+    stop("Unsupported type of 'workers': ", class(workers)[1])
   }
   stopifnot(length(workers) == 1, is.finite(workers), workers >= 1)
   workers
@@ -35,6 +39,7 @@ nbrOfWorkers.uniprocess <- function(evaluator) 1L
 nbrOfWorkers.multiprocess <- function(evaluator) {
   expr <- formals(evaluator)$workers
   workers <- eval(expr)
+  if (is.function(workers)) workers <- workers()
   stopifnot(length(workers) == 1, is.finite(workers), workers >= 1)
   workers
 }
@@ -43,6 +48,7 @@ nbrOfWorkers.multiprocess <- function(evaluator) {
 nbrOfWorkers.future <- function(evaluator) {
   expr <- formals(evaluator)$workers
   workers <- eval(expr)
+  if (is.function(workers)) workers <- workers()
   if (is.null(workers)) workers <- Inf
   workers
 }
