@@ -37,7 +37,7 @@ v0 <- local({
 })
 
 
-message("*** Globals - automatic ...")
+message("*** Globals & packages - automatic ...")
 
 for (strategy in supportedStrategies()) {
   message(sprintf("- Strategy: %s ...", strategy))
@@ -211,9 +211,78 @@ for (strategy in supportedStrategies()) {
   print(y)
   stopifnot(all.equal(y, v0))
 
+  
+  message("- Globals manually specified as named list - also with '...' ...")
+  x <- 1:10
+  y_truth <- x[2:3]
+  str(y_truth)
+  
+  ## Make sure it's possible to specify '...' as a global
+  sub <- function(x, ...) value(future(x[...], globals = c("x", "...")))
+  y <- sub(x, 2:3)
+  str(y)
+  stopifnot(identical(y, y_truth))
+
+  ## Make sure it's possible to specify '...' as a global (not just last)
+  ## Requires globals (> 0.11.0)
+  sub <- function(x, ...) value(future(x[...], globals = c("...", "x")))
+  if (packageVersion("globals") > "0.11.0")
+    y <- sub(x, 2:3)
+  str(y)
+  stopifnot(identical(y, y_truth))
+  
+  ## And if '...' is forgotten, it may give an error
+  sub <- function(x, ...) value(future(x[...], globals = "x"))
+  y <- tryCatch(sub(x, 2:3), error = identity)
+  str(y)
+  stopifnot((strategy %in% c("multisession") && inherits(y, "FutureError")) || identical(y, y_truth))
+
+
+  message("- Packages - manual ...")
+
+  ## Make sure 'iris', and thereby the 'datasets' package,
+  ## is not picked up as a global
+  unloadNamespace("datasets")
+  stopifnot(!"dataset" %in% loadedNamespaces(), !exists("iris", mode = "list"))
+  
+  ns %<-% {
+    unloadNamespace("datasets")
+    loadedNamespaces()
+  }
+  print(ns)
+  stopifnot(!"dataset" %in% ns)
+
+  res <- tryCatch({
+    f <- future({ iris })
+    v <- value(f)
+    print(head(v))
+  }, error = identity)
+  stopifnot(inherits(res, "error"))
+
+  f <- future({ iris }, packages = "datasets")
+  v <- value(f)
+  print(head(v))
+  
+  ns %<-% {
+    unloadNamespace("datasets")
+    loadedNamespaces()
+  }
+  print(ns)
+  stopifnot(!"dataset" %in% ns)
+
+  res <- tryCatch({
+    df %<-% { iris }
+    print(head(df))
+  }, error = identity)
+  stopifnot(inherits(res, "error"))
+
+  df %<-% { iris } %packages% "datasets"
+  print(head(df))
+  
   message(sprintf("- Strategy: %s ... DONE", strategy))
 }
 
 message("*** Globals - manually ... DONE")
 
+
 source("incl/end.R")
